/*

 ============================================================================
                   The Apache Software License, Version 1.1
 ============================================================================

 Copyright (C) 1999-2002 The Apache Software Foundation. All rights reserved.

 Redistribution and use in source and binary forms, with or without modifica-
 tion, are permitted provided that the following conditions are met:

 1. Redistributions of  source code must  retain the above copyright  notice,
    this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.

 3. The end-user documentation included with the redistribution, if any, must
    include  the following  acknowledgment:  "This product includes  software
    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
    Alternately, this  acknowledgment may  appear in the software itself,  if
    and wherever such third-party acknowledgments normally appear.

 4. The names "Apache Cocoon" and  "Apache Software Foundation" must  not  be
    used to  endorse or promote  products derived from  this software without
    prior written permission. For written permission, please contact
    apache@apache.org.

 5. Products  derived from this software may not  be called "Apache", nor may
    "Apache" appear  in their name,  without prior written permission  of the
    Apache Software Foundation.

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
 DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
 OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 This software  consists of voluntary contributions made  by many individuals
 on  behalf of the Apache Software  Foundation and was  originally created by
 Stefano Mazzocchi  <stefano@apache.org>. For more  information on the Apache
 Software Foundation, please see <http://www.apache.org/>.

*/
package org.apache.cocoon.components.language.markup.xsp;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.Reader;
import java.sql.CallableStatement;
import java.sql.ResultSet;
import java.io.InputStream;
import java.sql.Blob;
import java.sql.Clob;
import java.sql.Types;

/**
 * This is a helper class to remove redundant code in
 * esql pages
 *
 * based on the orginal esql.xsl
 * @author <a href="mailto:tcurdt@dff.st">Torsten Curdt</a>
 * @version CVS $Id: EsqlHelper.java,v 1.7.2.2 2002/07/05 14:43:14 haul Exp $
 */

public class EsqlHelper {


    /** returns byte array from BLOB
     */
    public final static byte[] getBlob(ResultSet set, String column) throws RuntimeException {
        
        byte[] result = null;
        try {
            result = EsqlHelper.getBlob(set,set.findColumn(column));
        } catch (Exception e) {
            throw new RuntimeException("Error getting blob data: " + e.getMessage());
        }
        return result;
    }

    /** returns byte array from BLOB
     */
    public final static byte[] getBlob(ResultSet set, int column) throws java.lang.Exception {
        
        InputStream reader = null;
        byte[] buffer = null;
    
        try {
            if (set.getMetaData().getColumnType(column)==java.sql.Types.BLOB) {
                Blob dbBlob = set.getBlob(column);
                int length = (int) dbBlob.length();
                reader = dbBlob.getBinaryStream();
                buffer = new byte[length];
                reader.read(buffer);
                reader.close();
                if (reader != null)
                    reader.close();
                if (buffer == null)
                    return null;
                return buffer;
            } else {           
                return set.getString(column).getBytes();
            }
        } catch ( Exception e) {
            throw new RuntimeException("Error getting blob data: " + e.getMessage());
        }
    }

    /** returns byte array from BLOB
     */
    public final static byte[] getBlob(CallableStatement cs, int column) throws java.lang.Exception {
        
        InputStream reader = null;
        byte[] buffer = null;
    
        try {
            Blob dbBlob = cs.getBlob(column);
            int length = (int) dbBlob.length();
            reader = dbBlob.getBinaryStream();
            buffer = new byte[length];
            reader.read(buffer);
            reader.close();
            if (reader != null)
                reader.close();
            if (buffer == null)
                return null;
            return buffer;
        } catch ( Exception e) {
            throw new RuntimeException("Error getting blob data: " + e.getMessage());
        }
    }

    /** returns Unicode encoded string from CLOB or String column 
     */
    public final static String getStringOrClob(ResultSet set, String column) throws RuntimeException {
        
        String result = null;
        try {
            result = EsqlHelper.getStringOrClob(set,set.findColumn(column));
        } catch (Exception e) {
            throw new RuntimeException("Error getting clob data: " + e.getMessage());
        }
        return result;
    }


    /** returns Unicode encoded string from CLOB or String column 
     */
    public final static String getStringOrClob(ResultSet set, int column) throws java.lang.Exception {
        
        Reader reader = null;
        char[] buffer = null;
    
        try {
            if (set.getMetaData().getColumnType(column)==java.sql.Types.CLOB) {
                Clob dbClob = set.getClob(column);
                int length = (int) dbClob.length();
                reader = new BufferedReader(dbClob.getCharacterStream());
                buffer = new char[length];
                reader.read(buffer);
                reader.close();
                if (reader != null)
                    reader.close();
                if (buffer == null)
                    return "";
                return new String(buffer);
            } else {           
                return set.getString(column);
            }
        } catch ( Exception e) {
            throw new RuntimeException("Error getting clob data: " + e.getMessage());
        }
    }

    /** returns Unicode encoded string from CLOB or String column 
     */
    public final static String getStringOrClob(CallableStatement cs, int column) throws java.lang.Exception {
        
        Reader reader = null;
        char[] buffer = null;
    
        try {
            Clob dbClob = cs.getClob(column);
            int length = (int) dbClob.length();
            reader = new BufferedReader(dbClob.getCharacterStream());
            buffer = new char[length];
            reader.read(buffer);
            reader.close();
            if (reader != null)
                reader.close();
            if (buffer == null)
                return "";
            return new String(buffer);
        } catch ( Exception e) {
            throw new RuntimeException("Error getting clob data: " + e.getMessage());
        }
    }


    /** returns ascii string from CLOB or String column 
     */
      public final static String getAscii(ResultSet set, String column) throws RuntimeException {

          String result = null;
          try {
              result = EsqlHelper.getAscii(set,set.findColumn(column));
          } catch (Exception e) {
              throw new RuntimeException("Error getting clob data: " + e.getMessage());
          }
          return result;
      }


    /** returns ascii string from CLOB or String column 
     */
      public final static String getAscii(ResultSet set, int column) {
        InputStream asciiStream = null;
        String result = null;

        try {
            if (set.getMetaData().getColumnType(column) == Types.CLOB) {
                byte[] buffer = null;
                Clob dbClob = set.getClob(column);
                int length = (int) dbClob.length();
                asciiStream = new BufferedInputStream(dbClob.getAsciiStream());
                buffer = new byte[length];
                asciiStream.read(buffer);
                asciiStream.close();
                result = (buffer!=null? new String(buffer) : null);
            } else {
                result = set.getString(column);
            }
        } catch (Exception e) {
            throw new RuntimeException("Error getting clob data: " + e.getMessage());
        } finally {
            if (asciiStream != null) try {asciiStream.close();} catch (Exception ase) {
                throw new RuntimeException("Error closing clob stream: " + ase.getMessage());
            }
        }
        
        return result;
      }

    /** returns ascii string from CLOB or String column 
     */
    public final static String getAscii(CallableStatement cs, int column) {
        InputStream asciiStream = null;
        String result = null;

        try {
            byte[] buffer = null;
            Clob dbClob = cs.getClob(column);
            int length = (int) dbClob.length();
            asciiStream = new BufferedInputStream(dbClob.getAsciiStream());
            buffer = new byte[length];
            asciiStream.read(buffer);
            asciiStream.close();
            result = (buffer!=null? new String(buffer) : null);
        } catch (Exception e) {
            throw new RuntimeException("Error getting clob data: " + e.getMessage());
        } finally {
            if (asciiStream != null) try {asciiStream.close();} catch (Exception ase) {
                throw new RuntimeException("Error closing clob stream: " + ase.getMessage());
            }
        }
        
        return result;
      }

      public final static String getStringFromByteArray(byte[] bytes, String encoding) {
        if (bytes != null) {
            try {
                return new String(bytes,encoding);
            } catch (java.io.UnsupportedEncodingException uee) {
                throw new RuntimeException("Unsupported Encoding Exception: " + uee.getMessage());
            }
        }
        else {
            return("");
        }
      }

}
