/*
 * $Header: /home/cvspublic/xml-cocoon2/src/scratchpad/src/org/apache/cocoon/validation/schematron/Attic/ValidationResult.java,v 1.2.2.2 2002/06/05 17:19:30 vgritsenko Exp $
 * $Revision: 1.2.2.2 $
 * $Date: 2002/06/05 17:19:30 $
 *
 * ====================================================================
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999-2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Commons", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 2001, Plotnix, Inc,
 * <http://www.plotnix.com/>.
 * For more information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

package org.apache.cocoon.validation.schematron;

import java.util.Iterator;
import java.util.List;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.SortedSet;
import java.util.TreeSet;

import org.apache.cocoon.validation.Violation;

/**
 * Represents the result of a Schematron validation process
 * 
 * <validationResult>
 *   list of <pattern> elements with <rule> subelements
 * </validationResult>
 *
 * @author  Ivelin Ivanov, ivelin@acm.org, ivelin@iname.com
 */
public class ValidationResult
{
  
  private ArrayList patterns_ = new ArrayList();
  
 
  /**
   * Returns a list of the patterns which
   * contain rules that failed during validation
   */
  public List getPattern()
  {
    return patterns_;
  }
  
  /**
   * Sets the list of the patterns which
   * contain rules that failed during validation
   */
  public void setPattern(Collection newPatterns)
  {
    patterns_ = new ArrayList();
    patterns_.addAll ( newPatterns );
  }
  
  /**
   * Add a pattern to the list
   */
  public void addPattern(Pattern p)
  {
    patterns_.add ( p );
  }
  
  public boolean isEmpty ()
  {
    return patterns_.isEmpty ();
  }
  
  /**
   * adds all errors to a sorted list
   * Key is XPath of each error location
   *
   * @todo implement this method !!!
   * @return SortedSet
   */
  public SortedSet toSortedSet()
  {
    
    if ( isEmpty() ) return null;
    
    SortedSet violations = new TreeSet();
    
    Iterator piter = getPattern().iterator();
    while (piter.hasNext ())
    {
      Pattern pattern = (Pattern)piter.next ();
      // System.out.println("Pattern name: " + pattern.getName() + ", id: " + pattern.getId() );
      Iterator ruleIter = pattern.getRule().iterator();
      while (ruleIter.hasNext ())
      {
        Rule rule = (Rule)ruleIter.next ();
        // System.out.println("    Rule name: " + rule.getContext() );

        Iterator assertIter = rule.getAssert().iterator();
        while (assertIter.hasNext ())
        {
          Assert anAssert = (Assert)assertIter.next ();

          // add the next assert to the violations set
          Violation v = new Violation();
          v.setPath( rule.getContext() );
          v.setMessage( anAssert.getMessage() );
          violations.add( v );
          // System.out.println("        Assert test: " + anAssert.getTest() + ", message: " + anAssert.getMessage() );
        }

        Iterator reportIter = rule.getReport().iterator();
        while (reportIter.hasNext ())
        {
          Report report = (Report)reportIter.next ();

          // add the next report to the violations set
          Violation v = new Violation();
          v.setPath( rule.getContext() );
          v.setMessage( report.getMessage() );
          violations.add( v );
          // System.out.println("        Report test: " + report.getTest() + ", message: " + report.getMessage() );
        }
      }
    }
    return violations;
  }
  
}
